require "spec_helper"
# require "active_record/core"
require "logstash_scribbler/message"

module LogstashScribbler
  describe Message do
    let(:message) {LogstashScribbler::Message.new({:error => "Something Bad", :message => nil, :object => [1,2,3]})}
    before do
      allow(message).to receive(:is_active_record?).and_return(false)
    end

    context "#log" do
      require 'logstash-event'
      it "should call Logtash::Event new" do
        expect(LogStash::Event).to receive(:new).once
        message.log
      end
    end

    context "#data" do
      it "should not return nil values" do
        expect(message.data).to eq({:error => "Something Bad", :object => [1,2,3]})
      end

      it "calls #namespaced_object when passed on object" do
        expect(message).to receive(:namespaced_object).and_return({:object => [1,2,3]})
        message.data
      end

      it "puts custom fields at top level" do
        options = {:error => "Something Bad", :custom_fields => {:name => "Sport Ngin", :error => "Something kind of Bad"}}
        message = LogstashScribbler::Message.new(options)
        expect(options[:custom_fields][:name]).to eq(message.data[:name])
        expect(options[:error]).to eq(message.data[:error])
      end

      it "keeps attribute precedence ((:error || :message) > :custom_fields > :object))" do
        options = {:message => "Normal Message", :custom_fields => { :message => "Custom Message", :object => [1]}, :object => [3,4,5]}
        message = LogstashScribbler::Message.new(options)
        allow(message).to receive(:is_active_record?).and_return(false)
        expect(message.data).to eq({ :message => "Normal Message", :object => [1] })
      end
    end

    context "#namespaced_object" do
      context "when passed non ActiveRecord::Base object" do
        it "returns a hash" do
          expect(message.namespaced_object).to be_a(Hash)
        end
        it "places object under the 'object' key" do
          expect(message.namespaced_object[:object]).to eq([1,2,3])
        end
      end

      context "when passed ActiveRecord::Base object" do
        before do
          allow(message).to receive(:is_active_record?).and_return(true)
        end
        it "creates hash from object" do
          expect(message.namespaced_object).to be_a(Hash)
        end
        it "prepends class name to instance variable names" do
          expect(message.namespaced_object.has_key?(:Array)).to eq(true)
        end
      end
    end

    context "#inspected_error" do
      context "when passed non Exception" do
        it "returns error" do
          expect(message.error).to eq("Something Bad")
        end
      end

      context "when passed Exception" do
        error = Exception.new
        message = LogstashScribbler::Message.new({:error => error})
        it "returns a hash of the exception" do
          expect(message.error).to be_a(Hash)
        end
      end
    end
  end

end
